@@ -24,7 +24,7 @@ echobox()
 
 echoerror()
 {
-  mesg="*****  $*  *****"
+  mesg=">>>>>  $*  <<<<<"
   equals=`echo "$mesg" | tr "\040-\377" "[=*]"`
   echo "$equals"
   echo "$mesg"
