@@ -19,6 +19,6 @@
 #define DUPFFMOD_H
 
 void DUPFFmul3mod(DUPFF ans, const DUPFF x, const DUPFF y, const DUPFF m);
-void DUPFFexpt3mod(DUPFF ans, const DUPFF x, const int power, const DUPFF m);
+void DUPFFexpt3mod(DUPFF ans, const DUPFF x, const long power, const DUPFF m);
 
 #endif
