@@ -26,7 +26,7 @@ void DUPFFprint(const DUPFF f)
   if (df < 0) { printf("0\n"); return; }
   if (df > 99) df = 99; /* avoid printing out too much */
   printf("[");
-  for (i=0; i<df ; i++) printf("%d, ", f->coeffs[i]);
+  for (i=0; i<df ; i++) printf("%lu, ", f->coeffs[i]);
   if (DUPFFdeg(f) > 99) printf(".... ,");
-  printf("%d]\n", f->coeffs[DUPFFdeg(f)]);
+  printf("%lu]\n", f->coeffs[DUPFFdeg(f)]);
 }
