@@ -1,6 +1,6 @@
 ;; cocoa5.el --- major mode for editing and running CoCoA
 
-;; Copyright (c) 2005-2017  John Abbott, Anna Bigatti
+;; Copyright (c) 2005-2019  John Abbott, Anna Bigatti
 ;; This file is part of the CoCoA distribution.
 ;; You are free to use any part of this file in your own programs.
 ;; Many thanks to Giovanna Roda and Giorgio Vecchiocattivi 
@@ -63,7 +63,7 @@
 ;; April 2017
 ;; send interrupt to CoCoA (C-c C-c) -- also in menu
 
-(setq cocoa5-mode-version "7 April 2017")
+(setq cocoa5-mode-version "9 Nov 2019")
 
 ;;==============================================================>
 ;;     CUSTOMIZATION in cocoa5.emacs
@@ -260,7 +260,8 @@
   (concat "\\(" cocoa5-lc-startkeywords "\\|" cocoa5-uc-startkeywords "\\)" ))
 
 (defconst cocoa5-endkeywords-list 
-  (concat "\\("  (concat "end\\(" cocoa5-lc-startkeywords "\\)")
+  (concat "\\([Uu]ntil\\)\\|"
+	  "\\("  (concat "end\\(" cocoa5-lc-startkeywords "\\)")
 	  "\\|"  (concat "End\\(" cocoa5-uc-startkeywords "\\)")
 	  "\\)"))
 
@@ -273,7 +274,6 @@
 (defconst cocoa5-beg-block-re (concat "\\<" cocoa5-startkeywords-list "\\>"))
 (defconst cocoa5-end-block-re
   (concat "\\<\\("
-	  "[Uu]ntil\\|"
 	  cocoa5-endkeywords-list
 	  "\\)\\>"))
 (defconst cocoa5-declaration-re "\\<\\(NIENTE_DA_FISSARE\\)\\>")
@@ -284,15 +284,14 @@
 (defconst cocoa5-noindent-re
 (concat "\\<\\("
 	cocoa5-endkeywords-list "\\|[Ee]xport\\|"
-	"[Ee]lse\\|[Ee]lif\\|[Uu]ntil\\|UponError\\|uponerror" ;; JAA replaced Elsif by Elif 20090512
+	"[Ee]lse\\|[Ee]lif\\|UponError\\|uponerror" ;; JAA replaced Elsif by Elif 20090512
 	"\\)\\>"))
 ;; ;; (defconst cocoa5-nosemi-re
 ;;   "\\<\\([Tt]hen\\|[Dd]o\\|[Ee]lse\\|UponError\\|uponerror\\)\\>")
 (defconst cocoa5-autoindent-lines-re
   (concat "\\<\\("
-	  cocoa5-startkeywords-list"\\|"
-	  cocoa5-endkeywords-list"\\|"
-	  cocoa5-noindent-re
+	  cocoa5-startkeywords-list
+	  "\\|" cocoa5-noindent-re
 	  "\\)\\>"))
 
 ;; ;;; Strings used to mark beginning and end of excluded text
@@ -309,8 +308,8 @@
 (if cocoa5-mode-syntax-table
     ()
   (setq cocoa5-mode-syntax-table (make-syntax-table))
-  (modify-syntax-entry ?( "()"  cocoa5-mode-syntax-table)
-  (modify-syntax-entry ?) ")("  cocoa5-mode-syntax-table)
+  (modify-syntax-entry ?\( "()"  cocoa5-mode-syntax-table)
+  (modify-syntax-entry ?\) ")("  cocoa5-mode-syntax-table)
 ;  (modify-syntax-entry ?* "." cocoa5-mode-syntax-table)
   (modify-syntax-entry ?+ "."    cocoa5-mode-syntax-table)
   (modify-syntax-entry ?= "."    cocoa5-mode-syntax-table)
@@ -372,7 +371,6 @@
    ; keywords
    (concat "\\<\\("
 	   cocoa5-startkeywords-list "\\|"
-	   cocoa5-endkeywords-list "\\|"
 	   cocoa5-noindent-re "\\|"
 	   "[Aa]nd\\|[Oo]r\\|"
 	   "[Bb]reak\\|[Cc]ontinue\\|[Cc]iao\\|[Dd]o\\|[Dd]escribe\\|[Ee]xit\\|"
@@ -916,7 +914,7 @@ column number the line should be indented to."
   (interactive)
   (while (and
 	  (cocoa5-last-start-or-end-keyword)
-	  (looking-at cocoa5-endkeywords-list))
+	  (looking-at cocoa5-end-block-re))
     (cocoa5-last-unended-begin)))
 
 (defun cocoa5-close-block ()
@@ -942,7 +940,7 @@ column number the line should be indented to."
   "Move point to start of last keyword (outside string/comment) before current point"
   (let ((keyword-regexp (concat 
 			 "\\<\\(" cocoa5-startkeywords-list "\\|"
-			 cocoa5-endkeywords-list "\\)\\>"
+			 cocoa5-end-block-re "\\)\\>"
 			 )))
     (while (and 
 	    (re-search-backward keyword-regexp)
@@ -1341,6 +1339,51 @@ Special commands:
   (run-hooks 'cocoa5-mode-hook)
 )
 
+;;-------------------------------------------------------------->
+;; cocoa5-send-string
+;;-------------------------------------------------------------->
+
+(defun cocoa5-send-string (cocoa5-input-line)
+  "Send string cocoa5-input-line as input to CoCoA in the other window.
+Note: if the *cocoa5* buffer has no process, CoCoA will be restarted
+automatically when this function is called.
+"
+  (interactive)
+  (cocoa5-shell-check-process)
+;;  (end-of-line)
+;;  (setq cocoa5-line-end (point))
+;;  (setq cocoa5-last-line-without-newline (eobp))
+;;  (beginning-of-line)
+;;  (setq cocoa5-line-beg (point))
+;;  (setq cocoa5-line-length (- cocoa5-line-end cocoa5-line-beg))
+;;  (setq cocoa5-line-blankp (= (skip-chars-forward " \t") cocoa5-line-length))
+;;  (if (not cocoa5-line-blankp)
+;;      (if (> cocoa5-line-length 999) ;; use cocoa5-send-region for lines over 999 chars
+;;					;then
+;;	  (let ((save-echo cocoa5-echo-source-region))
+;;	    (setq cocoa5-echo-source-region nil)
+;;	    (cocoa5-send-region cocoa5-line-beg cocoa5-line-end)
+;;	    (setq cocoa5-echo-source-region save-echo))
+					;else
+	(save-excursion
+;;	  (setq cocoa5-input-line (buffer-substring cocoa5-line-beg cocoa5-line-end))
+	  (setq cocoa5-source-buffer (current-buffer)) 
+	  (cocoa5-shell-check-window)
+	  (cocoa5-pop-to-buffer-comint)
+	  (goto-char (point-max))
+(move-beginning-of-line nil) (if (not (eq (point) (point-max))) (kill-line))
+	  (insert cocoa5-input-line)
+	  (comint-send-input)
+	  (cocoa5-pop-to-buffer-source)
+	  );;))
+;;  (end-of-line)
+;;  ;; if last line was non-empty and without newline, insert a newline
+;;  (if (and cocoa5-last-line-without-newline
+;;	   (not cocoa5-line-blankp))
+;;      (insert "\n"))  ;; Emacs will report error if the buffer is read-only
+;;  (skip-chars-forward " \t\n")
+)
+
 ;;-------------------------------------------------------------->
 ;; cocoa5-send-line
 ;;-------------------------------------------------------------->
@@ -1377,6 +1420,7 @@ automatically when this function is called.
 	  (cocoa5-shell-check-window)
 	  (cocoa5-pop-to-buffer-comint)
 	  (goto-char (point-max))
+(move-beginning-of-line nil) (if (not (eq (point) (point-max))) (kill-line))
 	  (insert cocoa5-input-line)
 	  (comint-send-input)
 	  (cocoa5-pop-to-buffer-source)
@@ -1404,6 +1448,7 @@ automatically when this function is called.
 ;    (pop-to-buffer "*cocoa5*")
     (cocoa5-pop-to-buffer-comint)
     (goto-char (point-max))
+(move-beginning-of-line nil)  (if (not (eq (point) (point-max))) (kill-line))
     (insert arg)
     (comint-send-input)
 ;    (switch-to-buffer-other-window cocoa5-file-buffer-name)
@@ -1615,6 +1660,7 @@ automatically when this function is called.
     (switch-to-buffer-other-window "*cocoa5*")
     (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
     (goto-char (point-max))
+(move-beginning-of-line nil)  (if (not (eq (point) (point-max))) (kill-line))
     (yank)
     (comint-send-input)
 ;  (switch-to-buffer-other-window cocoa5-file-buffer-name)
@@ -1715,6 +1761,7 @@ automatically when this function is called.
     (cocoa5-pop-to-buffer-comint)
     (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
     (goto-char (point-max))
+(move-beginning-of-line nil)  (if (not (eq (point) (point-max))) (kill-line))
     (insert cocoa5-shell-source-command)
     (if cocoa5-echo-source-region
 	(insert "\n" cocoa5-sourceregion-region) ;; output region
@@ -1722,6 +1769,8 @@ automatically when this function is called.
     (comint-send-input)
     (cocoa5-pop-to-buffer-source)
   )
+  (goto-char region-end)
+  (deactivate-mark)
 )
 
 ;;-------------------------------------------------------------->
@@ -1764,6 +1813,7 @@ automatically when this function is called.
     (cocoa5-pop-to-buffer-comint)
     (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
     (goto-char (point-max))
+(move-beginning-of-line nil)  (if (not (eq (point) (point-max))) (kill-line))
     (insert cocoa5-shell-source-command)
     (comint-send-input)
     (cocoa5-pop-to-buffer-source)
@@ -2118,6 +2168,7 @@ automatically when this function is called.
 ;    (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
     (goto-char (point-max))
 ;    (insert "debug ")
+(move-beginning-of-line nil)  (if (not (eq (point) (point-max))) (kill-line))
     (insert arg)
     (comint-send-input)
 ;    (switch-to-buffer-other-window cocoa5-file-buffer-name)
